import hashlib
import os
import time

from filewatch.file_observer import file_updated_subject
from filewatch.config import settings

class Watcher(object):
    def __init__(self):
        # A map of filepath keys to their last modified date
        self.files = {}

        self._first_run = True

        # Maintain a set of files seen to keep track of deleted files
        self._files_seen = set()

    def run(self, start_directory=None):
        """Continually check the filesytem until the process ends"""
        while True:
            self.perform_check(start_directory=start_directory)

            interval = float(settings.get('DEFAULT', 'interval'))
            time.sleep(interval)

    def perform_check(self, start_directory=None):
        """Run a check over filesytem.

        :note: If no start directory is supplied then we will begin walking
        from the current working directory.
        """
        if not start_directory:
            start_directory = os.getcwd()

        files_updated = []
        seen = set()

        for dirpath, dirnames, filenames in os.walk(start_directory):
            for filename in filenames:
                updated = False
                full_path = os.path.join(dirpath, filename)
                key = full_path

                file_modified = os.path.getmtime(full_path)

                try:
                    last_changed = self.files[key]
                    updated = last_changed < file_modified

                    if updated:
                        # Update file change time
                        self.files[key] = file_modified
                except KeyError:
                    # We have not seen this file before, add it our dict and
                    # broadcast
                    self.files[key] = file_modified
                    updated = not self._first_run

                if updated:
                    meta = {
                        'size': os.path.getsize(full_path),
                    }

                    files_updated.append((full_path, meta))
                seen.add(key)

        # Add deleted files with an empty meta dict
        for deleted_file_path in self._files_seen - seen:
            files_updated.append((deleted_file_path, None))

        if files_updated:
            file_updated_subject.notify(file_list=files_updated)

        # Keep track of state
        self._first_run = False
        self._files_seen = seen



class Subject(object):
    def __init__(self):
        self.observers = []

    def notify(self, *args, **kwargs):
        """Notify our observers about an event"""
        for observer in self.observers:
            observer.notify(*args, **kwargs)

    def register_observer(self, observer):
        """Keep track of our observer"""
        if observer not in self.observers:
            self.observers.append(observer)

    def remove_observer(self, observer):
        """Remove a single observer"""
        if observer in self.observers:
            self.observers.remove(observer)

    def remove_observers(self):
        """Remove all observers"""
        self.observers = []

class ObserverBase(object):
    """Base class for any observers of `Subject`"""
    def notify(self, *args, **kwargs):
        """Subject is updating all observers that an event has occured"""
        raise NotImplementedError()



from filewatch.observer import Subject

class FileUpdatedSubject(Subject):
    """Broadcast a list of updated files"""
    def notify(self, *args, **kwargs):
        file_list = kwargs['file_list']
        for observer in self.observers:
            # As we don't know what the observers will do to our filelist,
            # create a local copy so they all get the same list
            local_file_list = file_list[:]
            observer.notify(file_list=local_file_list)

file_updated_subject = FileUpdatedSubject()
# We only ever want one subject available, so remove the original class
# from the namespace
del FileUpdatedSubject


import ConfigParser
import os

def get_config(config_name, env_name=None):
    """Load a config file.

    We look in various locations for a config file, starting with the
    default one in the app's current directory. We then check the user's
    home directory before looking for the appropriate environment variable.
    """
    if not env_name:
        env_name = '%s_CONF' % config_name.split('.')[0]

    config = ConfigParser.RawConfigParser()

    path_to_config = os.path.join(os.path.dirname(__file__), config_name)

    home_path = os.path.expanduser('~/.%s' % config_name)
    paths = [path_to_config, home_path, ]

    if env_name in os.environ:
        paths.append(os.environ[env_name])

    config.read(paths)

    return config

settings = get_config(config_name='filewatch.ini')


#!/usr/bin/env python
from distutils.core import setup, Command
import os

from filewatch import VERSION

# you can also import from setuptools

class PyTest(Command):
    user_options = []
    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import sys,subprocess
        errno = subprocess.call([sys.executable, 'runtests.py'])
        raise SystemExit(errno)

current_dir = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(current_dir, 'README.rst'), 'rb') as fin:
    README = fin.read()

PACKAGE_NAME = 'filewatch'

tests_require = [
    'pytest',
]

setup(name=PACKAGE_NAME,
      version=VERSION,
      cmdclass = {'test': PyTest},
      description='Python File Watcher',
      package_data={PACKAGE_NAME: ['{}.ini'.format(PACKAGE_NAME), ], },
      long_description=README,
      author='Ben Emery',
      url='https://github.com/benemery/%s' % PACKAGE_NAME,
      download_url='https://github.com/benemery/%s/tarball/%s' % (VERSION, PACKAGE_NAME),
      packages=[PACKAGE_NAME, ],
      extras_require={
        'tests': tests_require,
    },
     )